// THE HOVGAARD KLAN
// PROJECT: HOVTEXT
//---------------------------------------------------------------------------
// NOTES........ :
// This is the form that does all the real work:
// It manages the tray-icons, contains the context-menu
// and does everything concerning the clipboard.
// The form itself is never shown.
//
//---------------------------------------------------------------------------

#ifndef ClipFormH
#define ClipFormH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Menus.hpp>
#include <ExtCtrls.hpp>
#include <Graphics.hpp>
#include <AppEvnts.hpp>
#include <vcl\Clipbrd.hpp>
#include <rope>
#include <string>
#include <vector>
#include <hash_set>
#include "lang.h"
//---------------------------------------------------------------------------

#define WM_TRAY_ICON_MESSAGE (WM_USER+1997)
//---------------------------------------------------------------------------
#define WM_TRAY_ICON_ID (9)


class TClipForm1 : public TForm, public TLangChangeNotify
{
__published:	// IDE-managed Components
  TPopupMenu *PopupMenu1;
  TMenuItem *Active1;
  TMenuItem *Exit1;
  TImage *ImageInactive;
  TImage *ImageActive;
  TMenuItem *About1;
  TMenuItem *Options1;
  TImage *ImageHotkey;
  TImage *AltImageInactive;
  TImage *AltImageActive;
  TImage *AltImageHotkey;
  TTimer *SetupTimer;
  void __fastcall Exit1Click(TObject *Sender);
  void __fastcall Active1Click(TObject *Sender);
  void __fastcall About1Click(TObject *Sender);
  void __fastcall Options1Click(TObject *Sender);
  void __fastcall SetupTimerTimer(TObject *Sender);

public:		// User declarations
  __fastcall TClipForm1(TComponent* Owner);
  __fastcall ~TClipForm1();
  void __fastcall ChangeIcons(short set=0);
  virtual void LangChange();

protected:
#pragma option push
#pragma option -w-inl
  BEGIN_MESSAGE_MAP
    MESSAGE_HANDLER(WM_TRAY_ICON_MESSAGE,TMessage,TrayIcon_HandleMessage)
    MESSAGE_HANDLER(WM_HOTKEY,TMessage,WMHotKey)
    MESSAGE_HANDLER( WM_CHANGECBCHAIN, TWMChangeCBChain, WMChangeCBChain )
    MESSAGE_HANDLER( WM_DRAWCLIPBOARD, TWMDrawClipboard, WMDrawClipboard )
  END_MESSAGE_MAP(TForm)
#pragma option pop

private:	// User declarations

//---------------------------------------------------------------------------
// WinAPI Clipboard-functions/Hooks 

  HWND m_hWndClipChain;
  void __fastcall WMChangeCBChain( TWMChangeCBChain& Message );
  void __fastcall WMDrawClipboard( TWMDrawClipboard& Message );


//---------------------------------------------------------------------------
// Types and functions for saving/restoring the clipboard:

  UINT CF_HTML;
  UINT CF_RICHTEXT;
  UINT CF_EXCEL;

  bool SetTextClipboard(const char *text);

  // Contains the clipboard-format and data for one format
  struct TSaveFmt
  {
    unsigned fmt;
    long size;
    std::string name, intro;
    void *data;
  };

  // A complete clipboard entry with several formats
  typedef std::vector <TSaveFmt> TSaveClipbd;

  bool HasFmt(TSaveClipbd cb1, UINT fmt);
  void * GetFmt(TSaveClipbd cb1, UINT fmt);
  long GetFmtSize(TSaveClipbd cb1, UINT fmt);
  bool _fastcall SaveClipboard(TSaveClipbd &SaveClip);
  bool _fastcall RestoreClipboard(const TSaveClipbd &SaveClip, bool onlytext=false);
  void _fastcall ClrSaveClip(TSaveClipbd &SaveClip);
  void _fastcall CopySaveClipboard(const TSaveClipbd &cbfrom, TSaveClipbd &cbto);
  bool _fastcall CmpSaveClipboardText(const TSaveClipbd &cb1, const TSaveClipbd &cb2);
  bool _fastcall ConvertSavedClipboard(const TSaveClipbd &SaveClip, std::string &resultstring);
  std::string _fastcall SaveClipText(const TSaveClipbd &SaveClip);

  // List of all the clipboard formats, whic have been registered
  std::hash_set<std::string> RegClipFmt;

  // The most recent clipboard (All formats), for restoring on shutdown/deactivation
  TSaveClipbd GlobalSaveClip;

  // Clipboard history
  std::vector <TSaveClipbd> History;
  short HisPut, HisGet, HisMax;


  bool __fastcall ConvertedClipboard(std::string &result);
  bool __fastcall ConvertClip();

//---------------------------------------------------------------------------
// Handling of the tray-icon
  void TrayIcon_Create (const char *tipText,HICON hIcon);
  void TrayIcon_Delete(void);
  void TrayIcon_HandleMessage(TMessage &Message);
  void TrayIcon_ModifyTip(const char *tipText);
  void TrayIcon_ModifyIcon(HICON hIcon);
  void TrayIcon_ShowBalloonTip(const char *szMsg,
                const char *szTitle, unsigned uTimeout, long dwInfoFlags);

//---------------------------------------------------------------------------



  void __fastcall SetActive(bool active);
  void __fastcall SetHotKey(int Nr, AnsiString name);
  void __fastcall UnsetHotKey(int Nr);
  void __fastcall WMHotKey(TMessage &Message);
  bool HotKeySet[3];
  bool ConvertOnCopy, StripWS, StripCR,  OnlyCopyLinks,  RemoveDoubles;

};


//---------------------------------------------------------------------------
extern PACKAGE TClipForm1 *ClipForm1;
//---------------------------------------------------------------------------
#endif